//+------------------------------------------------------------------+
//|                                                                  |
//|  R-FATL-SATL-Adaptive v.1.0                                      |
//|  Copyright  2009 Riccardo "Rich" Cap                            |
//|  email: richcap.rc@gmail.com                                     |
//|                                                                  |
//|                                                                  |
//|  This indicator draws  FATL and SATL curves (and respective      |
//|  reference lines (RFTL and RSTL) re-calculated over a window of  |
//|  length 'length' every 'filterPersistenceBars' bars              |
//|  Optionally curves can be joined with an interpolation           |
//|  This indicator needs 'R-MESA_Cutoff_frequencies' indicator as   |
//|  well as R-MESA.ex4 and R-DF.ex4 libraries installed             |
//|                                                                  |
//|  Permission to use, copy, modify, and distribute this software   |
//|  for any purpose without fee is hereby granted, provided that    |
//|  this entire notice is included in all copies of any software    |
//|  which is or includes a copy or modification of this software    |
//|  and in all copies of the supporting documentation for such      |
//|  software.                                                       |
//|  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS    |
//|  OR IMPLIED WARRANTY.  IN PARTICULAR, NEITHER THE AUTHOR DOES    |
//|  NOT MAKE ANY REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING  |
//|  THE MERCHANTABILITY OF THIS SOFTWARE OR ITS FITNESS FOR ANY     |
//|  PARTICULAR PURPOSE.                                             |
//|                                                                  |
//+------------------------------------------------------------------+

#import "R-DF.ex4"
bool DF(
   double& h[],         // Impulse response of final filter [numtaps] (OUTPUT)
   int numtaps,         // Number of filter coefficients (INPUT)
   int numband,         // Number of bands in filter specification (INPUT)
   double& bands[],     // User-specified band edges [2 * numband] (INPUT)
   double& des[],       // User-specified band responses [numband] (INPUT)
   double& weight[],    // User-specified error weights [numband] (INPUT)
   int type);           // Type of filter (INPUT)
int PMcCl_filterOrd(
   double deltap,       //  Ripple for first band (passband, for a lowpass filter)
   double deltas,       //  Ripple for second band (stopband, for a lowpass filter)
   double fp,           //  Cutoff frequency for first band (startof transition band)
   double fs);          //  Cutoff frequency for second band (end of transition band)
#import

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Red            // FATL
#property indicator_color2 Orange         // RFTL
#property indicator_color3 LimeGreen      // SATL
#property indicator_color4 Aqua           // RSTL

//---- buffers
double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];

// External parameters 
extern int degree=150;                 // Degree of autocorrelation for MESA (Maximum Entropy Spectrum Analysis)
extern int length=400;                 // Number of previous Bars to perform MESA calculation
extern double max_period=140;          // Upper bound for SATL cutoff period (1/frequency)
extern double min_period=20;           // Lower bound for FATL cutoff period
extern double satl_min_period=60.0;    // Lower bound for SATL (upper bound for FATL is 80% of "*")
extern datetime initial_time=D'1970.01.01 00:00'; // if backwardBars=0, get it from initial_time
extern int filterPersistenceBars=100;  // The SATL and FATL cutoff periods are re-calculated every "*" bars
extern int backwardBars=0;             // The indicator is drawn backard "*" bars       ** Check if == 0 ** (MESA_Cutoff_frequencies problems??

extern bool rectify=false;             // by Default fatl/satl curves are not rectified to be continuous between two different filter sets
                                       // In this case must be paid attention with strategies that look at crossovers and slope

// FIR
double FATL_FIR[];                     // fatl filter coefficients are stored here
double SATL_FIR[];                     // satl filter coefficients are stored here
int FATL_filterSize=0;
int SATL_filterSize=0;
int RFTL_period=0;                     // if 0 it is set half the stopband period FATL_D1     ** Check **
int RSTL_period=0;                     // if 0 it is set half the stopband period SATL_D1     ** Check **
int filterIterator=0;      

double SATL_P1=100.0;                    // cutoff period SATL_P1
double SATL_D1=60.0;                     // cutoff period SATL_D1 (transition)
double FATL_P1=25.0;                     // cutoff period FATL_P1
double FATL_D1=23.0;                     // cutoff period FATL_D1 (transition)

// other
double fatl_diff=0.0, satl_diff=0.0;   
static bool secondInstanceRunning=false;
static datetime prevtime=0;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  { 
   
   // sets semaphore to check if is already running. Wanted to create a singleton instance , it is the closest way
   if (!GlobalVariableCheck("ACTF_ON"))                
      GlobalVariableSet("ACTF_ON", 0.0);

   double semaphore = GlobalVariableGet("ACTF_ON");
   Print ("Semaphore value: ", semaphore);
   if ( semaphore == 1.0)
   {
      Print ("Indicator is already running");
      secondInstanceRunning=true;             
      //return (10);                            // uncomment to avoid second instance to run
   }
   else
   {
      Print ("New instance of indicator");
      GlobalVariableSet("ACTF_ON", 1.0);
      secondInstanceRunning=false;
   }
   
   // sets number of initial bars   
   if (backwardBars==0)
   {
      if (initial_time == D'1970.01.01 00:00')
         backwardBars=0;
      else
         backwardBars=iBarShift(NULL,0,initial_time, true)-1;        
   }   
   if (backwardBars==-1)
   {
      Print ("Initial time not valid");
      return (20);            
   }
   else  Print("shift of bar with open time ",TimeToStr(initial_time)," is ",backwardBars);
   
   //---- indicators
   IndicatorDigits(Digits+2);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexDrawBegin(0,0);
   SetIndexDrawBegin(1,0);
   SetIndexDrawBegin(2,0);
   SetIndexDrawBegin(3,0);
   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexBuffer(2,Buffer3);
   SetIndexBuffer(3,Buffer4);
   SetIndexLabel(0,"FATL");
   SetIndexLabel(1,"RFTL");
   SetIndexLabel(2,"SATL");
   SetIndexLabel(3,"RSTL");

   //  Clean screen
   ObjectsDeleteAll(0, OBJ_VLINE);
   ObjectsDeleteAll(0, OBJ_TEXT);
   
   filterIterator=0;          // iterator to change filters (based on MESA) every N steps (N=filterPersistenceBars)
   filterPersistenceBars--;  // To have a real N step period, it is necessary to put N-1 this value
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   // remove semaphore
   if (!secondInstanceRunning) 
      GlobalVariableSet("ACTF_ON", 0.0);
     
   return(0);
}
  
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{ 
   int i, pos, counted_bars=IndicatorCounted();

   if(!newPeriodOpening())
   {
      //Print("Not the beginning of a new period");
      return(0);  // Not the beginning of a new period
   } 

   if (secondInstanceRunning)
   {
         //return(1);           // uncomment to avoid second instance to run
   }  
   
   if(Bars<=backwardBars)
   {
      Print("Too many Bars backwards, please stay under ",Bars," Bars");
      return(0);
   }   
   i=backwardBars;      
   if (counted_bars>Bars-backwardBars-2) 
      i=Bars-counted_bars-2;
         
   //Print ("Pos:",pos," - counted bars:", counted_bars,"- Bars:", Bars, " -Indicator counted:",IndicatorCounted()," - Now: ",TimeToStr(TimeCurrent()));  
   while(i>=0)
   {
      getFilters(i);
      drawIndicators(i, counted_bars);
      i--;
   }
      
   return(0);
}
  
  
bool drawIndicators(int shift, int counted_bars)
{ 
   int i, j, MA_Period;
   double fatl, satl, rftl, rstl;
 
   
   // DRAW FATL
   if (FATL_filterSize>0) 
   {
      // evaluate bias between old and new filter by calculating previous value with new filter coeff
      if (filterIterator == filterPersistenceBars)  // Filter has changed
      {

         fatl=0.0;
         if (rectify)
         {
            for (j=0; j<FATL_filterSize; j++) 
            {
               fatl=fatl+FATL_FIR[j]*Close[shift+j+2];   
            }
            fatl_diff = fatl - Buffer1[shift+2];
         }
         else 
            fatl_diff=0.0;
            
         if (MathAbs(fatl_diff) > 1000000.0) fatl_diff =0.0;          // Buffer[shift+1] was uninitialized
         
         // draw vertical bar to put in evidence new filter coefficients
         ObjectCreate(StringConcatenate("FilterChange",TimeToStr(iTime(NULL,0,shift+2))),OBJ_VLINE, 0, iTime(NULL,0,shift+2),0.0);
         
         // print new cutoff values
         if(!ObjectCreate(StringConcatenate("fatl_abel",TimeToStr(iTime(NULL,0,shift+2))),OBJ_TEXT, 0, iTime(NULL,0,shift+2),fatl))
         {
            Print("error: can't create text_object! code #",GetLastError());
         }
         else    // print FATL_P1 and D1 values
         {
            string fatl_text= StringConcatenate("P1: ",DoubleToStr(FATL_P1,1)," - D1: ",DoubleToStr(FATL_D1,1));
            ObjectSetText(StringConcatenate("fatl_abel",TimeToStr(iTime(NULL,0,shift+2))), fatl_text, 8, "Arial", indicator_color1);
         }     
            
  
      }
   
      fatl=0.0;
      for (j=0; j<FATL_filterSize; j++) 
      {
         fatl=fatl+FATL_FIR[j]*Close[shift+j+1];   
      }
      // calculate new fatl and add bias with a linear descent to zero
      Buffer1[shift+1]=fatl-fatl_diff*filterIterator/(1.0*filterPersistenceBars);     
   } 
   else 
      Print("Fatl filter cannot run.");
   
   fatl=Buffer1[shift+1];
   
      
   // Draw RFTL as a simple moving average of FATL                               // is it OK ??
   MA_Period=RFTL_period;
   if ( Buffer1[shift+MA_Period] < 1000000.0 )            // check if most far buffer value is initialized otherwise skip
   { 
      double sum=0;
      for(i=0;i<MA_Period;i++)
         sum+=Buffer1[shift+1+i];
      Buffer2[shift+1]=sum/MA_Period;  
   }
    
   rftl=Buffer2[shift+1];                                                         // ???????????
   //-----------------------------------------   
    
      
   // Draw SATL      
   if (SATL_filterSize>0) 
   {
      // evaluate bias between old and new filter by calculating previous value with new filter coeff
      if (filterIterator == filterPersistenceBars)  // Filter has changed
      {
         satl=0.0;
         if (rectify)
         {
            for (j=0; j<SATL_filterSize; j++) 
            {
               satl=satl+SATL_FIR[j]*Close[shift+j+2];   
            }
            satl_diff = satl - Buffer3[shift+2];
         }
         else 
            satl_diff=0.0;
         
         if (MathAbs(satl_diff) > 1000000.0) satl_diff =0.0;          // Buffer[shift+1] was uninitialized
         
         // print new cutoff values
         if(!ObjectCreate(StringConcatenate("satl_label",TimeToStr(iTime(NULL,0,shift+2))),OBJ_TEXT, 0, iTime(NULL,0,shift+2),satl))
         {
            Print("error: can't create text_object! code #",GetLastError());
         }
         else    // print SATL_P1 and D1 values
         {
            string satl_text= StringConcatenate("P1: ",DoubleToStr(SATL_P1,1)," - D1: ",DoubleToStr(SATL_D1,1));
            ObjectSetText(StringConcatenate("satl_label",TimeToStr(iTime(NULL,0,shift+2))), satl_text, 8, "Arial", indicator_color3);
         }     
      }
      
      satl=0.0;
      for (j=0; j<SATL_filterSize; j++) 
      {
         satl=satl+SATL_FIR[j]*Close[shift+j+1];   
      }
      // calculate new satl and add bias with a linear descent to zero
      Buffer3[shift+1]=satl-satl_diff*filterIterator/(1.0*filterPersistenceBars);
   } 
   else 
      Print("Satl filter cannot run.");
   satl=Buffer3[shift+1]; 
   
   
     
   // Draw RSTL as a simple moving average of SATL                            // is it OK ??
   MA_Period=RSTL_period;
   if ( Buffer3[shift+MA_Period] < 1000000.0 )            // check if most far buffer value is initialized otherwise skip
   { 
      sum=0;
      for(i=0;i<MA_Period;i++)
         sum+=Buffer3[shift+1+i];
      Buffer4[shift+1]=sum/MA_Period;  
   }
   rstl=Buffer4[shift+1];                                                       // ???????????
   //-----------------------------------------   
   
   //Print("fatl: ",DoubleToStr(fatl,6)," - rftl: ",DoubleToStr(rftl,6), " - satl: ",DoubleToStr(satl,6)," - rstl: ",DoubleToStr(rstl,6), " - Bar's Time: ", TimeToStr(iTime(NULL,0,shift+1)));
   return(true);
}
  
  
 // Evaluate cutoff frequencies for each shift
bool getFilters(int shift)
{    
   // Common filter parameters
   double common_A1 = 40.0;            // stopband attenuation in dB
   double common_ripple = 0.8;         // passband ripple in dB
   
   // Other Custom MESA indicator parameters
   int bkwdBars=backwardBars+3;
   bool smoothed=true;
   
   // Check if filters must be calculated or not
   if ( filterIterator == 0)
      filterIterator=filterPersistenceBars; 
   else  
   {    
      filterIterator--;
      return (false);
   }
   
   // Calculate new cutoff frequencies
   double new_SATL_P1= iCustom(NULL,0, "R-MESA_Cutoff_frequencies",degree,length,max_period,min_period,satl_min_period,bkwdBars,smoothed,0,shift+1);
   double new_SATL_D1= iCustom(NULL,0, "R-MESA_Cutoff_frequencies",degree,length,max_period,min_period,satl_min_period,bkwdBars,smoothed,1,shift+1);
   double new_FATL_P1= iCustom(NULL,0, "R-MESA_Cutoff_frequencies",degree,length,max_period,min_period,satl_min_period,bkwdBars,smoothed,2,shift+1);
   double new_FATL_D1= iCustom(NULL,0, "R-MESA_Cutoff_frequencies",degree,length,max_period,min_period,satl_min_period,bkwdBars,smoothed,3,shift+1);
   //Print ("Backward Bars:",bkwdBars," - Shift: ", shift," - Bar's Time: ", TimeToStr(iTime(NULL,0,shift+1))," - new_SATL_P1:", new_SATL_P1,"  new_SATL_D1:", new_SATL_D1, "  new_FATL_P1:", new_FATL_P1, "  new_FATL_D1:", new_FATL_D1);
   
   // if no usable cutoff frequency are available, old ones are used
   //un-initialized buffer value are outside this range (2147483647)
   if (new_SATL_P1 > 0.0 && new_SATL_P1 < 1000000.0 && new_FATL_P1 > 0.0 && new_FATL_P1 < 1000000.0 )    
   {    
      SATL_P1=new_SATL_P1;
      SATL_D1=new_SATL_D1;
      FATL_P1=new_FATL_P1;
      FATL_D1=new_FATL_D1;
      Print ("Shift: ", shift," - Bar's Time: ", TimeToStr(iTime(NULL,0,shift+1))," - SATL_P1:", SATL_P1,"  SATL_D1:", SATL_D1, "  FATL_P1:", FATL_P1, "  FATL_D1:", FATL_D1);
   
   }
   else
      Print("Shift: ", shift," - Bar's Time: ", TimeToStr(iTime(NULL,0,shift+1))," - Unable to evaluate cutoff frequencies, using old or predefined:   SATL_P1:", SATL_P1,"  SATL_D1:", SATL_D1, "  FATL_P1:", FATL_P1, "  FATL_D1:", FATL_D1);
   
   
   if (RFTL_period == 0) RFTL_period = FATL_D1/2;
   if (RSTL_period == 0) RSTL_period = SATL_D1/2;
   
   // Calculate new filter coefficients
   FATL_filterSize = getLowPassFilter(FATL_FIR, FATL_P1, FATL_D1, common_ripple, common_A1);
   SATL_filterSize = getLowPassFilter(SATL_FIR, SATL_P1, SATL_D1, common_ripple, common_A1);
   
   return(true);
}
  


/*
 * Get Lowpass filter coefficients
 */ 
  
int getLowPassFilter(
   double& FIR[],
   double P1,
   double D1,
   double ripple,
   double A1 )
{  
   double bands[20];
   double des[10];
   double weight[10];
   int filterOrder=0;
   int filterSize=0;
  
   // Create Filter Specs   
   double fp = 1.0/P1;           // cutoff frequency passband (corresponding to cutoff period P1)
   double fs = 1.0/D1;           // cutoff frequency stopband (corresponding to cutoff period D1) 
   double deltap=0.5*(1.0 - MathPow(10.0, -0.05*ripple));
   double deltas = MathPow(10.0, -0.05*A1);
   
   bands[0]=0.0;                 // Beginning of first band
   bands[1]= fp;                 // end of first band
   des[0]=1.0;                   // gain in first band
   weight[0]=1.0;                // weight in first band
   
   bands[2]=fs;
   bands[3]=1.0/2.0;
   des[1]=0.0;                   // gain in second band (stopband)  
   weight[1]=deltap/deltas;      // weight in second band (stopband): it is 1.0 if deltap=deltas
   
   /*
   bands[4]=1.0/20.0;
   bands[5]=0.5;
   des[2]=0.001;                 // gain in third band (stopband)  -40dB
   weight[2]=1.0;                // weight in second band (stopband)
   */
   
   // Estimate filter order 
   filterOrder=PMcCl_filterOrd(deltap,deltas,fp,fs);   
   ArrayResize(FIR,filterOrder);
   
   
   // Create Filter Coefficients
   if (!DF( FIR, filterOrder, 2, bands, des, weight, 1))     // Type of filter BANDPASS with 2 bands
      Print("Unable to create digital filter");
   
   filterSize=filterOrder/2+filterOrder%2;
   
   // Some info about filter
   double gain=0.0;
   for (int j=0; j<filterSize; j++) 
      gain+=FIR[j];      
   //Print("Number of coefficients: ",filterSize," - Filter Order: ", filterOrder," - Gain: ",gain);                     
   
   return (filterSize);
}
  
bool newPeriodOpening()
{
   if(prevtime == Time[0]) return(false);
   prevtime = Time[0];
   return(true);
}
  
   
//-------------------------------------------------------------------+